@@ -1,30 +1,162 @@
 import collections, re
-from .common import log, build_where_msg, tohuman, print_example, \
-                    enhance_exceptions
+from .common import log, build_where_msg, tohuman, \
+                    enhance_exceptions, print_example, constant
 
-Where = collections.namedtuple('Where', ['start_lineno',
-                                         'end_lineno',
-                                         'filepath'])
+from .parser import ExampleParser
+from .options import Options
+
+class Where(object):
+    def __init__(self, start_lineno, end_lineno, filepath):
+        self.start_lineno = start_lineno
+        self.end_lineno = end_lineno
+        self.filepath = filepath
+
+    def as_tuple(self):
+        return (self.start_lineno, self.end_lineno, self.filepath)
+
+    def __iter__(self):
+        return iter(self.as_tuple())
+
+    def __repr__(self):
+        return repr(self.as_tuple())
+
+class Example(object):
+    '''
+    The unit work of byexample: the example.
+
+    It represents a example found in <where> by the <finder> that should be
+    parsed by the <parser> and executed later by the <runnner>.
+
+    The piece of text found is given by <snippet> and <extracted_str>:
+    the code the be executed and the expected output.
+
+    These two are *incomplete* and further processing is need by <parser>
+
+    >>> from byexample.finder import _build_fake_example
+
+    >>> example = _build_fake_example('f()', '42', fully_parsed=False)
+    >>> example
+    Example (not parsed yet) [python] in file file.md, lines 0-2
+
+    The example is incomplete or not fully parsed because the <snippet> may
+    not be the code in its final state; the expected regex doesn't exist nor
+    the options.
+
+    >>> example.source
+    <...>
+    AttributeError: 'Example' object has no attribute 'source'
+
+    >>> example.expected
+    <...>
+    AttributeError: 'Example' object has no attribute 'expected'
+
+    >>> example.options
+    <...>
+    AttributeError: 'Example' object has no attribute 'options'
+
+    After the completion those attributes should be defined
+
+    >>> example.parse_yourself()
+    Example [python] in file file.md, lines 0-2
+
+    >>> example.source
+    'f()\n'
+
+    >>> example.expected.str
+    '42'
+
+    >>> example.options
+    {'norm_ws': False, 'rm': [], 'tags': True}
+
+    '''
+    def __init__(self, finder, runner, parser, snippet, expected_str, indent, where):
+        self.finder, self.runner, self.parser = finder, runner, parser
+        self.snippet, self.expected_str, self.indentation = snippet, expected_str, indent
+
+        self.start_lineno, self.end_lineno, self.filepath = where
+
+        self.fully_parsed = False
+
+    def parse_yourself(self, concerns=None):
+        if self.fully_parsed:
+            raise ValueError("You cannot parse/build an example twice: " + \
+                             repr(self))
+
+        where = Where(self.start_lineno, self.end_lineno, self.filepath)
+        self.parser.parse(self, concerns)
+        self.fully_parsed = True
+
+        return self
+
+    def __repr__(self):
+        f = "" if self.fully_parsed else "(not parsed yet) "
+        return "Example %s[%s] in file %s, lines %i-%i" % (
+                        f,
+                        self.runner.language,
+                        self.filepath, self.start_lineno, self.end_lineno)
+
+def _build_fake_example(snippet, expected, language='python', start_lineno=0,
+                            specific=False, fully_parsed=True, opts=None):
+    class R: pass    # <- fake runner instance
+    R.language = language # <- language of the example
+
+    class F: pass    # <- fake finder instance
+    F.specific = specific # <- is finder specific?
+
+    # fake a parser
+    parser = ExampleParser(0, 'utf8', Options())
+    parser.language = language
+
+    # fake the options parsed by the parser
+    if opts == None:
+        opts = {'norm_ws': False, 'tags': True, 'rm': []}
+    parser.extract_options = lambda x: opts
+
+    # fake the start-end lines where the example "was found"
+    end_lineno = start_lineno
+    end_lineno += (snippet + '\n' + expected).count('\n') + 1
+    where = Where(start_lineno, end_lineno, 'file.md')
+
+    # create it
+    e = Example(F, R, parser, snippet, expected, "", where)
+
+    # parse it (fake, of course)
+    if fully_parsed:
+        e = e.parse_yourself(concerns=None)
+
+    return e
 
 class ExampleHarvest(object):
     '''
-          Finding process    Parsing process         Run process
-    ----------\                           example
-    | foo     |     example match    ............ . . .
-    |         |      -----------     : > 1 + 2 } snippet  =>  1 + 2 ----> runner
-    | > 1 + 2 |      | > 1 + 2 |     :                                      |
-    | 3       |  =>  | 3       |  => : 3 } expected       =>  3 == ?? <-- output
-    |         |      -----------     :.......... .. . .       compare
-    | bar     |      -----------                           done by checker
-    | > 1 + 3 |  =>  | > 1 + 3 |                                |
-    | 4       |      | 4       |                                |--> PASS/FAIL
-    :         :      -----------                                      report
-                                                                  done by reporter
+                  Finding process             Parsing process
+    ----------\                      example                 example (parsed)
+    | foo     |      a match       (not parsed)         ............ . . .
+    |         |    -----------    -----------           : 1 + 2 } source
+    | > 1 + 2 |    | > 1 + 2 |    | 1 + 2 } snippet  => :
+    | 3       | => | 3       | => | 3 } expected_str    : 3 } expected (regex)
+    |         |    -----------    -----------           :
+    | bar     |    -----------                          : options extracted too
+    | > 1 + 3 | => | > 1 + 3 |                          :.......... .. . .
+    | 4       |    | 4       |                               |         |
+    :         :    -----------          /                    V         |
+                                       /        executed { 1 + 2       |
+                                      /         by runner    |         |
+                                     |                  . .  V . . . . V . . .
+                              Run    |    compare done  :  output   output   :
+                            process  |    by expected   :   got    expected  :
+                                     |      object      : .  . . . . . . . . :
+                                     |                            |
+                                     |                            V
+                                     |     report done by     PASS/FAIL
+                                     \     differ and by
+                                      \       reporter
+                                       \         .
     '''
     def __init__(self, allowed_languages, registry, verbosity,
-                        options, **unused):
+                        options, use_colors, **unused):
         self.allowed_languages = allowed_languages
         self.verbosity = verbosity
+        self.use_colors = use_colors
         self.available_finders = registry['finders'].values()
 
         self.parser_by_language = registry['parsers']
@@ -32,6 +164,9 @@ class ExampleHarvest(object):
 
         self.options = options
 
+    def __repr__(self):
+        return 'Example Harvester'
+
     def get_examples_from_file(self, filepath):
         with open(filepath, 'rtU') as f:
             string = f.read()
@@ -40,15 +175,18 @@ class ExampleHarvest(object):
 
     def get_examples_from_string(self, string, filepath='<string>'):
         all_examples = []
+        log("Finding examples...", self.verbosity-1)
         for finder in self.available_finders:
-            examples = self.get_examples_using(finder, string, filepath)
+            examples = self.get_example_using(finder, string, filepath)
             all_examples.extend(examples)
 
         # sort the examples in the same order
         # that they were found in the file/string.
         all_examples.sort(key=lambda this: this.start_lineno)
 
-        return self.check_example_overlap(all_examples, filepath)
+        all_examples = self.check_example_overlap(all_examples, filepath)
+
+        return all_examples
 
     def check_example_overlap(self, examples, filepath):
         r'''
@@ -59,30 +197,15 @@ class ExampleHarvest(object):
         start_lineno attribute. In case of two examples with the same
         start_lineno, they must be sorted in reverse order by their end_lineno
 
-        Before going deeper, let's create a helper function to build examples:
+        Before going deeper, let's use a helper function to build examples:
 
-            >>> # helper function to create examples
-            >>> from byexample.parser import Example
-            >>> def build_example(source, expected, language, start_lineno, specific=False):
-            ...    class R: pass    # <- fake runner instance
-            ...    R.language = language # <- language of the example
-            ...
-            ...    class F: pass    # <- fake finder instance
-            ...    F.specific = specific # <- is finder specific?
-            ...
-            ...    end_lineno = start_lineno
-            ...    end_lineno += (source + '\n' + expected).count('\n') + 1
-            ...    return Example(R,
-            ...                   None, F,
-            ...                   start_lineno, end_lineno,
-            ...                   None, None,
-            ...                   source, expected)
+            >>> from byexample.finder import _build_fake_example as build_example
 
-        And we create a harvester to play with it:
+        And create a harvester to play with it:
 
             >>> from byexample.finder import ExampleHarvest
             >>> f = ExampleHarvest([], dict((k, {}) for k in \
-            ...                   ('parsers', 'finders', 'runners')), 0, None)
+            ...                   ('parsers', 'finders', 'runners')), 0, 0, None)
 
         Okay, back to the check_example_overlap documentation,
         given the examples sorted in that way, a collision is detected if
@@ -92,16 +215,15 @@ class ExampleHarvest(object):
 
               1...........5.....7       1...........5       1...........5
               |  example  |             |  example  |       |  example  |
-              |     example     |           |  ex  |        |  example  |
+              |     example     |           |  ex |         |  example  |
                  |   example    |       1..2......4         1...........5
               1..2..............7
 
         What do we do will depend of the content of the examples, not only of
         the type of collision:
 
-         - if both examples have the same language and one example is contained
-           inside the other (Collision 2 or Collision 3), we drop the example
-           found by a generic finder.
+         - if one example is contained inside the other (Collision 2 or
+           Collision 3), we drop the example found by a generic finder.
 
            This scenario can happen when a generic finder like FencedMatchFinder
            finds a interpreter-session-like for the same language.
@@ -110,13 +232,22 @@ class ExampleHarvest(object):
                 ```python
                 >>> 1 + 2
                 3
-
                 ```
-           So we remove the example of the generic finder (FencedMatchFinder) as
-           we assume that it may have more precise information (the correct
-           source code and the correct expected string)
 
-           >>> A = build_example('>>> 1 + 2', '3', 'python', start_lineno=1)
+           In that case both examples have the same language.
+           But there is a good case when that may not hold: using a markdown
+           for one language (due its highligth syntax) and writting the real
+           example inside using a prompt:
+                ```shell
+                >>> 1 + 2
+                3
+                ```
+
+           So we remove the example of the generic finder (FencedMatchFinder) as
+           we assume that the specific may have more precise information
+           (the correct source code and the correct expected string)
+
+           >>> A = build_example('1 + 2', '3', 'python', start_lineno=1)
            >>> B = build_example('1 + 2', '3', 'python', start_lineno=1, specific=True)
 
            >>> examples = f.check_example_overlap([A, B], 'foo.rst')
@@ -124,7 +255,10 @@ class ExampleHarvest(object):
            1
 
            >>> examples
-           [Example(<...> source='1 + 2', <...>)]
+           [Example [python] in file <...>, lines 1-3]
+
+           >>> examples[0].snippet
+           '1 + 2'
 
 
          - if both examples have the same source code, expected string and
@@ -140,7 +274,7 @@ class ExampleHarvest(object):
            1
 
            >>> examples
-           [Example(<...>start_lineno=1, end_lineno=3<...>)]
+           [Example [python] in file <...>, lines 1-3]
 
          - if the example A has a source code that contains the example B's
            source code and all the B's code is inside in A's one, we will
@@ -162,18 +296,21 @@ class ExampleHarvest(object):
            1
 
            >>> examples
-           [Example(<...> source='# python comment\n1 + 2', <...>)]
+           [Example [python] in file <...>, lines 1-4]
+
+           >>> examples[0].snippet
+           '# python comment\n1 + 2'
 
            The border case is when both examples have the same source code but
            are examples of two different languages.
-           It is too ambiguo so we fail
+           It is too ambiguous so we fail
 
            >>> A = build_example('# python comment', '', 'python', start_lineno=1)
            >>> B = build_example('# python comment', '', 'shell', start_lineno=1)
 
            >>> examples = f.check_example_overlap([A, B], 'foo.rst')
            Traceback<...>
-           ValueError: In foo.rst, examples at line 1 (found by <...>) and at line 1 (found by <...>) overlap each other.
+           ValueError: In foo.rst, examples at lines 1-3 (found by <...>) and at lines 1-3 (found by <...>) overlap each other.
 
         - in any other case, we fail too
 
@@ -184,14 +321,14 @@ class ExampleHarvest(object):
 
            >>> examples = f.check_example_overlap([A, B], 'foo.rst')
            Traceback<...>
-           ValueError: In foo.rst, examples at line 1 (found by <...>) and at line 1 (found by <...>) overlap each other.
+           ValueError: In foo.rst, examples at lines 1-4 (found by <...>) and at lines 1-5 (found by <...>) overlap each other.
 
            >>> A = build_example('a\nb', '', 'sh', start_lineno=1)      # span 2
            >>> B = build_example('b\nc', '', 'sh', start_lineno=2)      # span 2
 
            >>> examples = f.check_example_overlap([A, B], 'foo.rst')
            Traceback<...>
-           ValueError: In foo.rst, examples at line 2 (found by <...>) and at line 1 (found by <...>) overlap each other.
+           ValueError: In foo.rst, examples at lines 2-5 (found by <...>) and at lines 1-4 (found by <...>) overlap each other.
 
           For a Collision 2:
 
@@ -200,7 +337,7 @@ class ExampleHarvest(object):
 
            >>> examples = f.check_example_overlap([A, B], 'foo.rst')
            Traceback<...>
-           ValueError: In foo.rst, examples at line 2 (found by <...>) and at line 1 (found by <...>) overlap each other.
+           ValueError: In foo.rst, examples at lines 2-4 (found by <...>) and at lines 1-5 (found by <...>) overlap each other.
 
           For a Collision 3:
 
@@ -209,7 +346,7 @@ class ExampleHarvest(object):
 
            >>> examples = f.check_example_overlap([A, B], 'foo.rst')
            Traceback<...>
-           ValueError: In foo.rst, examples at line 1 (found by <...>) and at line 1 (found by <...>) overlap each other.
+           ValueError: In foo.rst, examples at lines 1-4 (found by <...>) and at lines 1-4 (found by <...>) overlap each other.
 
         '''
 
@@ -229,7 +366,6 @@ class ExampleHarvest(object):
                                     not collision_type_3)
 
                 any_collision = collision_type_1 or collision_type_2 or collision_type_3
-
                 if not any_collision:
                     prev = example
                     continue
@@ -239,64 +375,93 @@ class ExampleHarvest(object):
                 curr_where = Where(example.start_lineno, example.end_lineno, filepath)
                 prev_where = Where(prev.start_lineno, prev.end_lineno, filepath)
 
+                self._log_debug(" * Collision Type (1/2/3): %s/%s/%s\n"        \
+                                " * Languages (prev/current): %s/%s\n"         \
+                                " * Specific? (prev/current): %s/%s\n"
+                                    % (collision_type_1, collision_type_2,
+                                        collision_type_3, prev.runner.language,
+                                        example.runner.language, prev.finder.specific,
+                                        example.finder.specific), curr_where)
+                print_example(prev, self.use_colors, self.verbosity-3)
+                print_example(example, self.use_colors, self.verbosity-3)
+
+                if collision_type_2 or collision_type_3:
+                    if example.finder.specific != prev.finder.specific:
+                        if example.finder.specific:
+                            del examples[i-1]
+                            _where = prev_where
+                        else:
+                            del examples[i]
+                            _where = curr_where
+
+                        self._log_drop("generic/specific overlap", _where)
+                        break
+
                 if same_language:
-                    if collision_type_2 or collision_type_3:
-                        if example.finder.specific != prev.finder.specific:
-                            if example.finder.specific:
-                                del examples[i-1]
-                                _where = prev_where
-                            else:
-                                del examples[i]
-                                _where = curr_where
-
-                            self._log_drop("generic/specific overlap", _where)
-                            break
-
-                    if example.source == prev.source and \
-                            example.expected == prev.expected:
+                    if example.snippet == prev.snippet and \
+                            example.expected_str == prev.expected_str:
 
                         del examples[i]
                         self._log_drop("duplicated examples", curr_where)
                         break
 
                 else:
-                    if example.source == prev.source:
+                    if example.snippet == prev.snippet:
                         pass # too ambiguous
 
-                    elif example.source in prev.source:
+                    elif example.snippet in prev.snippet:
+                        self._log_drop("inner example (%s)" %
+                                examples[i].runner.language, curr_where)
                         del examples[i]
-                        self._log_drop("inner example", curr_where)
                         break
 
-                    elif prev.source in example.source:
+                    elif prev.snippet in example.snippet:
+                        self._log_drop("inner example (%s)" %
+                                examples[i-1].runner.language, prev_where)
                         del examples[i-1]
-                        self._log_drop("inner example", prev_where)
                         break
 
-                msg = "In %s, examples at line %i (found by %s) and " +\
-                      "at line %i (found by %s) overlap each other."
-                msg = msg % (filepath, example.start_lineno,
-                             example.finder, prev.start_lineno,
+                msg = "In %s, examples at lines %i-%i (found by %s) and " +\
+                      "at lines %i-%i (found by %s) overlap each other."
+                msg = msg % (filepath, example.start_lineno, example.end_lineno,
+                             example.finder, prev.start_lineno, prev.end_lineno,
                              prev.finder)
                 raise ValueError(msg)
 
+        if self.verbosity-1 >= 0:
+            log("Examples after removing any overlapping", self.verbosity-1)
+            for finder in set(e.finder for e in examples):
+                log("File '%s': %i examples [%s]" % (
+                                    filepath,
+                                    len([e for e in examples if e.finder==finder]),
+                                    str(finder)),
+                                    self.verbosity-1)
+
+        if self.verbosity-2 >= 0:
+            for e in examples:
+                print_example(e, self.use_colors, 0)
+            print("")
         return examples
 
     def _log_drop(self, reason, where):
-        log(build_where_msg(where, self, "Dropped example: " + reason),
-                self.verbosity-2)
+        self._log_debug(" => Dropped example: " + reason, where)
 
-    def get_examples_using(self, finder, string, filepath='<string>'):
+    def _log_debug(self, what, where):
+        log(build_where_msg(where, self, what), self.verbosity-3)
+
+    def get_example_using(self, finder, string, filepath='<string>'):
         charno = 0
         start_lineno = 1  # humans tend to count from 1
         examples = []
 
         for match in finder.get_matches(string):
             example_str = string[match.start():match.end()]
+            if example_str.endswith('\n'):
+                example_str = example_str[:-1]
 
             # start_lineno and end_lineno are inclusive
             start_lineno += string[charno:match.start()].count('\n')
-            end_lineno = start_lineno + example_str.count('\n') - 1
+            end_lineno = start_lineno + example_str.count('\n')
 
             # update charno here
             charno = match.start()
@@ -331,24 +496,27 @@ class ExampleHarvest(object):
                 # save the indentation here
                 indent = match.group('indent')
 
-                # then, get the snippet (runneable code) and the expected (the string)
-                # from the example_str
+                # then, get the source (runneable code) and the expected (the string)
                 snippet, expected = finder.get_snippet_and_expected(match, where)
 
+                if expected == None:
+                    expected = ""
+
             with enhance_exceptions(where, parser):
                 # perfect, we have everything to build an example
-                example = parser.build_example(snippet, expected, indent,
-                                                        runner, finder, where)
-
+                example = Example(finder, runner, parser,
+                                        snippet, expected, indent, where)
                 examples.append(example)
 
 
         log("File '%s': %i examples [%s]" % (filepath, len(examples), str(finder)),
-                                            self.verbosity-1)
+                                            self.verbosity-2)
 
         return examples
 
 
+
+
 class ExampleFinder(object):
     specific = True
 
@@ -372,24 +540,28 @@ class ExampleFinder(object):
         r'''
         Given an example string, remove its indentation
 
-            >>> from byexample.finder import ExampleFinder
+            >>> from byexample.finder import ExampleFinder, Where
             >>> mfinder = ExampleFinder(0, 'utf8'); mfinder.target = 'python-prompt'
             >>> check_and_remove_indent = mfinder.check_and_remove_indent
-            >>> check_and_remove_indent('  >>> 1 + 2\n  3', '  ', (1, 2, 'foo.rst'))
+
+            >>> where = Where(1, 2, 'foo.rst')
+            >>> check_and_remove_indent('  >>> 1 + 2\n  3', '  ', where)
             '>>> 1 + 2\n3'
 
-        If the string contains a line with a lower level of indentation,
-        raise an exception.
+            >>> where
+            (1, 2, 'foo.rst')
 
-            >>> check_and_remove_indent('  >>> 1 + 2\n3', '  ', (1, 2, 'foo.rst'))
-            Traceback (most recent call last):
-            <...>
-            ValueError: The line 2 is misaligned (wrong indentation). Expected at least 2 spaces.
-            001   >>> 1 + 2
-            002 3
+        If the string contains a line with a lower level of indentation,
+        truncate the example at that point and ignore the rest.
+
+            >>> check_and_remove_indent('  >>> 1 + 2\n3', '  ', where)
+            '>>> 1 + 2'
+
+            >>> where
+            (1, 1, 'foo.rst')
 
         The only exception to this are the empty lines
-            >>> check_and_remove_indent('  >>> 1 + 2\n\n  3', '  ', (1, 2, 'foo.rst'))
+            >>> check_and_remove_indent('  >>> 1 + 2\n\n  3', '  ', where)
             '>>> 1 + 2\n\n3'
 
         '''
@@ -400,28 +572,24 @@ class ExampleFinder(object):
         indent_stripped = []
         for lineno, line in enumerate(lines):
             if not line.startswith(indent) and line:
-                msg = 'The line %i is misaligned (wrong indentation). ' +\
-                      'Expected at least %i spaces.\n%s'
-
-                radio = 2
-                context = lines[max(lineno-radio, 0):lineno+radio+1]
-                context = [("%03i " % (i + start_lineno)) + l
-                            for i, l in enumerate(context, max(lineno-radio, 0))]
-
-                msg = msg % (start_lineno + lineno,
-                                len(indent),
-                                '\n'.join(context))
-                raise ValueError(msg)
+                # shrink the example and update the new end line number
+                where.end_lineno = start_lineno + lineno - 1
+                break
 
             indent_stripped.append(line[len(indent):])
 
+        if not indent_stripped:
+            raise ValueError("Inconsistent state.")
+
         return '\n'.join(indent_stripped)
 
     def check_keep_matching(self, example_str, match):
         r'''
         Given an example string, try to apply the match again.
+
         This is a health-check intended to be used after a call to
-        'check_and_remove_indent'
+        'check_and_remove_indent', 'remove_spurious_endings' and
+        other processing functions.
 
             >>> from byexample.finder import ExampleFinder
             >>> import re
@@ -457,12 +625,15 @@ class ExampleFinder(object):
         new_match = match.re.match(example_str)
         if not new_match:
             msg = 'The regex does not match the example after ' +\
-                  'removing the indentation (check_and_remove_indent). '
+                  'processing it (filtering & removing) done by ' +\
+                  'ExampleFinder.get_snippet_and_expected method. '
 
             raise ValueError(msg)
 
         if new_match.start() != 0 or new_match.end() != len(example_str):
-            msg = '%i bytes were left out after removing the indentation (check_and_remove_indent). ' +\
+            msg = '%i bytes were left out after processing it ' +\
+                  '(filtering & removing) done by '+\
+                  'ExampleFinder.get_snippet_and_expected method. ' +\
                   'Dropped bytes at the %s of example:\n%s\n'
 
             if new_match.start() != 0:
@@ -492,9 +663,53 @@ class ExampleFinder(object):
         # update the example_str removing any indentation;
         example_str = self.check_and_remove_indent(example_str, indent, where)
 
+        example_str = self.remove_spurious_endings(example_str)
+
         # check that we still can find the example
         # (allow to generate a new match)
         match = self.check_keep_matching(example_str, match)
 
         # finally, return the updated snippet and expected strings
         return match.group('snippet'), match.group('expected')
+
+    def remove_spurious_endings(self, expected_str):
+        '''
+            >>> from byexample.finder import ExampleFinder
+            >>> mfinder = ExampleFinder(0, 'utf8'); mfinder.target = 'python-prompt'
+            >>> remove_spurious_endings = mfinder.remove_spurious_endings
+
+            A spurious ending is a particular string that is found
+            in the last line.
+
+            It must be at the begin of the last line:
+            >>> remove_spurious_endings('>>> 1 + 2\n3\n```')
+            '>>> 1 + 2\n3\n'
+
+            And it must be the only thing in the line (with the exception
+            of any trailing whitespace):
+
+            >>> remove_spurious_endings('>>> 1 + 2\n3\n-->  ')
+            '>>> 1 + 2\n3\n'
+
+            But if the last line has something else, it will not be removed:
+            >>> remove_spurious_endings('>>> foo()\n<-- hello\nworld -->')
+            '>>> foo()\n<-- hello\nworld -->'
+
+            It will not be removed even it the last line has leading
+            whitespace:
+            >>> remove_spurious_endings('>>> foo()\n<-- hello\n   -->')
+            '>>> foo()\n<-- hello\n   -->'
+            '''
+
+        return self.spurious_ending_regex().sub('\n', expected_str)
+
+    @constant
+    def spurious_ending_regex(self):
+        # Be at the start of the line and be the last line of the expected
+        # string
+        endings = '|'.join(re.escape(e) for e in sorted(self.spurious_endings()))
+        return re.compile(r"(\n|\A)(%s)[ ]*\n?\Z" % endings, re.DOTALL | re.MULTILINE)
+
+    def spurious_endings(self):
+        return {"'''", '-->', '```', '~~~'}
+
