@@ -1,14 +1,39 @@
-import sys, pkgutil, inspect, pprint
+import sys, pkgutil, inspect, pprint, os
 
 from .options import Options, OptionParser
 from .runner import ExampleRunner
 from .finder import ExampleHarvest, ExampleFinder
 from .executor import FileExecutor
-from .checker import Checker
+from .differ import Differ
 from .parser import ExampleParser
 from .concern import Concern, ConcernComposite
 from .common import log
 
+def are_tty_colors_supported(output):
+    def get_colors():
+        try:
+            import curses
+            curses.setupterm()
+            return int(curses.tigetnum('colors'))
+        except:
+            # assume that we have a terminal with color support
+            return 8
+
+    # assume that colors are enabled by default iff:
+    # - the output is a TTY terminal (the output is not a redirection
+    #   to a file or pipe for example)
+    # - the terminal name (TERM) has not the 'm' suffix like xterm-m
+    #   (see term(7) for more terminal names conventions)
+    # - ncurses queries the terminal and it says that colors are supported
+    #   and it has 8 or more colors.
+    #
+    # the order is important because, for example, the value of TERM is
+    # more relevant: the terminal may have support for colors but if TERM
+    # has the -m suffix, we should honor that; the user may relay on this.
+    return output.isatty() and \
+           'm' not in os.getenv('TERM', '').split('-') and \
+           get_colors() >= 8
+
 def is_a(target_class, key_attr):
     '''
     Returns a function that will return True if its argument
@@ -40,10 +65,14 @@ def load_modules(dirnames, cfg):
             module = importer.find_module(name).load_module(name)
         except Exception as e:
             log("From '%s' loading '%s'...failed: %s" % (path, name, str(e)),
-                                                        verbosity-2)
+                                                        verbosity)
             continue
 
-        log("From '%s' loaded '%s'" % (path, name), verbosity-1)
+        stability = getattr(module, 'stability', 'undefined')
+        if stability not in ('experimental', 'unstable', 'stable', 'deprecated'):
+            stability = 'experimental/%s?' % str(stability)
+
+        log("From '%s' loaded '%s' (%s)" % (path, name, stability), verbosity-1)
         for klass, key, what in [(ExampleRunner, 'language', 'runners'),
                                  (ExampleParser, 'language', 'parsers'),
                                  (ExampleFinder, 'target', 'finders'),
@@ -105,20 +134,30 @@ def _float_zero_to_none(x):
 
     return float(x)
 
+def geometry(x):
+    lines, columns = [int(v.strip()) for v in str(x).split('x')]
+    if lines < 0 or columns < 0:
+        raise ValueError("Invalid geometry %s" % x)
+
+    return (lines, columns)
+
 def get_default_options_parser(cmdline_args):
     options_parser = OptionParser()
     options_parser.add_flag("fail-fast", help="if an example fails, fail and stop all the execution.")
     options_parser.add_flag("norm-ws", help="ignore the amount of whitespaces.")
     options_parser.add_flag("pass", help="run the example but do not check its output.")
     options_parser.add_flag("skip", help="do not run the example.")
-    options_parser.add_flag("capture", help="enable the capture tags <...>.")
+    options_parser.add_flag("tags", help="enable the tags <...>.")
     options_parser.add_flag("enhance-diff", help="improve how the diff are shown.")
     options_parser.add_flag("interact", help="interact with the runner/interpreter manually if an example fails.")
-    options_parser.add_argument("+timeout", type=int, help="timeout in seconds to complete the example.")
+    options_parser.add_argument("+rm", action='append', help="remove a character from the got and expected strings.")
+    options_parser.add_argument("+timeout", type=float, help="timeout in seconds to complete the example.")
     options_parser.add_argument("+diff", choices=['none', 'unified', 'ndiff', 'context'],
                                         help="select diff algorithm.")
     options_parser.add_argument("+delaybeforesend", type=_float_zero_to_none,
                                     help="delay in seconds before sending a line to an runner/interpreter; 0 disable this (default).")
+    options_parser.add_argument("+geometry", type=geometry,
+                                    help="number of lines and columns of the terminal of the form LxC (default to 24x80).")
 
     return options_parser
 
@@ -130,17 +169,21 @@ def get_options(args, cfg):
                         'norm_ws': False,
                         'pass': False,
                         'skip': False,
-                        'capture': True,
+                        'tags': True,
+                        'rm': [],
                         'enhance_diff': args.enhance_diff,
                         'interact': args.interact,
                         'timeout': args.timeout,
                         'diff': args.diff,
                         'delaybeforesend': None,
+                        'shebangs': args.shebangs,
+                        'geometry': (24, 80)
                         })
     log("Options (cmdline): %s" % options, cfg['verbosity']-2)
 
     # create a parser for the rest of the options.
     optparser = get_default_options_parser(args)
+    options['optparser'] = optparser
 
     # we parse the argument 'options' to allow the user to change
     # some options.
@@ -162,7 +205,6 @@ def get_options(args, cfg):
     #
     # Because this, we pass these to the rest of the system to be used and
     # completed later
-    cfg['optparser'] = optparser
     cfg['options']= options
 
     log("Options (cmdline + --options): %s" % options, cfg['verbosity']-2)
@@ -172,16 +214,35 @@ def show_options(cfg, registry, allowed_languages):
     parsers = [p for p in registry['parsers'].values()
                if p.language in allowed_languages]
 
+    concerns = [c for c in registry['concerns'].values()]
+
     def _title(t, nl=True):
         print(('\n' + t) if nl else t)
         print("-" * len(t))
 
     _title("byexample's options", nl=False)
-    cfg['optparser'].print_help()
+    cfg['options']['optparser'].print_help()
+
+    for concern in concerns:
+        _title("%s's specific options" % concern.target)
+        concern.get_extended_option_parser(parent_parser=None).print_help()
+
     for parser in parsers:
         _title("%s's specific options" % parser.language)
         parser.get_extended_option_parser(parent_parser=None).print_help()
 
+def extend_option_parser_with_concerns(cfg, registry):
+    concerns = [c for c in registry['concerns'].values()]
+
+    # join the concerns' option parser into one single parser
+    # starting from the byexample's one
+    optparser = cfg['options']['optparser']
+    for concern in concerns:
+        optparser = concern.get_extended_option_parser(optparser)
+
+    cfg['options']['optparser'] = optparser
+    return optparser
+
 def extend_options_with_language_specific(cfg, registry, allowed_languages):
     parsers = [p for p in registry['parsers'].values()
                if p.language in allowed_languages]
@@ -209,8 +270,8 @@ def init(args):
 
     options = get_options(args, cfg)
 
-    # if the output is not atty, disable the color anyways
-    cfg['use_colors'] &= cfg['output'].isatty()
+    # if the output has not color support, disable the color anyways
+    cfg['use_colors'] &= are_tty_colors_supported(cfg['output'])
 
     registry = load_modules(args.modules_dirs, cfg)
 
@@ -223,6 +284,11 @@ def init(args):
         show_options(cfg, registry, allowed_languages)
         sys.exit(0)
 
+    # extend the option parser with all the parsers of the concerns.
+    # do this *after* showing the options so we can show each parser's opt
+    # separately
+    extend_option_parser_with_concerns(cfg, registry)
+
     # now that we know what languages are allowed, extend the options
     # for them
     extend_options_with_language_specific(cfg, registry, allowed_languages)
@@ -235,9 +301,9 @@ def init(args):
 
     concerns = ConcernComposite(registry, **cfg)
 
-    checker  = Checker(**cfg)
+    differ = Differ(**cfg)
 
     harvester = ExampleHarvest(allowed_languages, registry, **cfg)
-    executor  = FileExecutor(concerns, checker, **cfg)
+    executor  = FileExecutor(concerns, differ, **cfg)
 
     return testfiles, harvester, executor, options
