@@ -1,8 +1,10 @@
-import traceback, time, os
+import traceback, time, os, sys
 from byexample.common import colored, highlight_syntax
 from byexample.concern import Concern
 from doctest import _indent
 
+stability = 'unstable'
+
 try:
     from tqdm import tqdm
     if os.environ.get('BYEXAMPLE_PROGRESS_ASCII'):
@@ -51,17 +53,16 @@ class SimpleReporter(Concern):
     def _update(self, x):
         pass
 
-    def start_run(self, examples, runners, filepath):
-        self.examples = examples
+    def start(self, examples, runners, filepath):
         self.num_examples = len(examples)
         self.examplenro = 0
         self.filepath = filepath
         self.begin = time.time()
 
-        self.fail = self.good = self.aborted_or_crashed = self.skipped = 0
+        self.fail = self.good = self.skipped = 0
 
-    def end_run(self, failed, user_aborted, crashed):
-        if not self.examples:
+    def finish(self, failed, user_aborted, crashed, broken, timedout):
+        if self.num_examples == 0:
             if self.verbosity >= 1:
                 self._write("File %s, no test found\n" % self.filepath)
             return
@@ -82,7 +83,7 @@ class SimpleReporter(Concern):
 
         ran_number = self.examplenro
         tot_number = self.num_examples
-        if user_aborted or crashed:
+        if user_aborted or crashed or broken or timedout:
             status_str = colored("[ABORT]", 'red', self.use_colors)
         elif failed:
             status_str = colored("[FAIL]", 'red', self.use_colors)
@@ -98,11 +99,9 @@ class SimpleReporter(Concern):
         self._write(msg)
 
     def skip_example(self, example, options):
-        self.examplenro += 1
         self.skipped += 1
 
     def start_example(self, example, options):
-        self.examplenro += 1
         self.current_merged_flags = options
 
     def start_interact(self, example, options):
@@ -110,6 +109,20 @@ class SimpleReporter(Concern):
         self._write("Starting interactive session.\n")
         self._write("Escape character is '^]'.\n")
 
+    def timedout(self, example, exception):
+        self._write('\n')
+
+        msg = 'Execution timedout at example %i of %i.\n' % (
+                                    self.examplenro, self.num_examples)
+        msg += 'This could be because the example just ran too slow (try add more time\n' + \
+               'with +timeout=<n>) or the example is "syntactically incorrect" and\n' + \
+               'the interpreter hang (may be you forgot a parenthesis or something like that?).\n'
+
+        if exception.output:
+            msg += 'This is the last output obtained:\n%s\n' % str(exception.output)
+
+        self._print_error_header(example)
+        self._write(msg)
 
     def user_aborted(self, example):
         self._write('\n')
@@ -118,28 +131,64 @@ class SimpleReporter(Concern):
                                     self.examplenro, self.num_examples)
         self._print_error_header(example)
         self._write(msg)
-        self.aborted_or_crashed += 1
 
     def crashed(self, example, exception):
         self._write('\n')
 
-        msg = 'Execution of example %i of %i crashed.\n%s' % (
+        tb = ''.join(traceback.format_tb(self._get_traceback(exception)))
+        ex = '%s: %s' % (str(exception.__class__.__name__), str(exception))
+        msg = 'Execution of example %i of %i crashed.\n%s\n%s\n' % (
                                     self.examplenro, self.num_examples,
-                                    traceback.format_exc(exception))
+                                    tb, ex)
         self._print_error_header(example)
         self._write(msg)
-        self.aborted_or_crashed += 1
 
-    def success(self, example, got, checker):
+    def start_parse(self, example, options):
+        self.current_parsing_example = example
+
+    def finish_parse(self, example, options, exception):
+        self.examplenro += 1
+
+        if exception == None:
+            return
+
+        self._write('\n')
+
+        ex = '%s: %s' % (str(exception.__class__.__name__), str(exception))
+        if self.verbosity >= 1:
+            tb = ''.join(traceback.format_tb(self._get_traceback(exception)))
+            ex = '\n'.join([tb, ex])
+
+        msg = 'Parse of example %i of %i failed.\n%s\n' % (
+                                    self.examplenro, self.num_examples,
+                                    ex)
+        self._print_error_header(self.current_parsing_example)
+        self._write(msg)
+
+    def finish_interact(self, exception):
+        if exception == None:
+            return
+
+        self._write('\n')
+
+        ex = '%s: %s' % (str(exception.__class__.__name__), str(exception))
+        if self.verbosity >= 1:
+            tb = ''.join(traceback.format_tb(self._get_traceback(exception)))
+            ex = '\n'.join([tb, ex])
+
+        msg = 'Interactive session failed.\n%s\n' % (ex)
+        self._write(msg)
+
+    def success(self, example, got, differ):
         self._update(1)
         self.good += 1
 
-    def failure(self, example, got, checker):
+    def failure(self, example, got, differ):
         self._update(1)
         self._write("\n")
 
         self._print_error_header(example)
-        diff = checker.output_difference(example, got, self.current_merged_flags,
+        diff = differ.output_difference(example, got, self.current_merged_flags,
                                          self.use_colors)
         self._write(diff)
         self._write('\n')
@@ -158,6 +207,12 @@ class SimpleReporter(Concern):
         self._write("Failed example:\n")
         self._write(_indent(highlight_syntax(example, self.use_colors)))
 
+    def _get_traceback(self, exception):
+        if hasattr(exception, '__traceback__'):
+            return exception.__traceback__
+        else:
+            return sys.exc_info()[2]
+
 class ProgressBarReporter(SimpleReporter):
     target = None # progress
 
@@ -175,8 +230,8 @@ class ProgressBarReporter(SimpleReporter):
     def _update(self, x):
         self.bar.update(x)
 
-    def start_run(self, examples, runners, filepath):
-        SimpleReporter.start_run(self, examples, runners, filepath)
+    def start(self, examples, runners, filepath):
+        SimpleReporter.start(self, examples, runners, filepath)
 
         bar_format = '{desc} |{bar}| [{n_fmt}/{total_fmt}{postfix}]'
         self.bar = tqdm(total=len(examples), file=self.output,
@@ -185,9 +240,9 @@ class ProgressBarReporter(SimpleReporter):
                              disable=None # means disable if the output is not TTY
                              )
 
-    def end_run(self, failed, user_aborted, crashed):
+    def finish(self, failed, user_aborted, crashed, broken, timedout):
         self.bar.close()
-        SimpleReporter.end_run(self, failed, user_aborted, crashed)
+        SimpleReporter.finish(self, failed, user_aborted, crashed, broken, timedout)
 
     def start_example(self, example, options):
         SimpleReporter.start_example(self, example, options)
