@@ -7,7 +7,6 @@ Example:
   hello<...>world
 
   ```python
-
   j = 2
   for i in range(4):
       j += i
@@ -22,9 +21,11 @@ Example:
 
 import re, pexpect, sys, time
 from byexample.common import log, constant
-from byexample.parser import ExampleParser
+from byexample.parser import ExampleParser, ExtendOptionParserMixin
 from byexample.finder import ExampleFinder
-from byexample.runner import ExampleRunner, PexepctMixin
+from byexample.runner import ExampleRunner, PexepctMixin, ShebangTemplate
+
+stability = 'stable'
 
 class PythonPromptFinder(ExampleFinder):
     target = 'python-prompt'
@@ -150,6 +151,10 @@ class PythonParser(ExampleParser):
     _opts_re_for_noncomp = _example_options_string_regex_for(False)
     _opts_re_for_comp = _example_options_string_regex_for(True)
 
+    def __init__(self, *args, **kw):
+        ExampleParser.__init__(self, *args, **kw)
+        self._optparser_extended_by_comp_mode_cache = None
+
     def example_options_string_regex(self):
         return self._opts_re_for_comp if self.compatibility_mode \
                 else self._opts_re_for_noncomp
@@ -166,7 +171,7 @@ class PythonParser(ExampleParser):
         if getattr(self, 'compatibility_mode', True):
             parser.add_flag("NORMALIZE_WHITESPACE", help="[doctest] alias for +norm-ws.")
             parser.add_flag("SKIP", help="[doctest] alias for +skip.")
-            parser.add_flag("ELLIPSIS", help="[doctest] enables the ... capture.")
+            parser.add_flag("ELLIPSIS", help="[doctest] enables the ... wildcard.")
             parser.add_flag("DONT_ACCEPT_BLANKLINE", help="[doctest] take <BLANKLINE> as literal.")
             parser.add_flag("DONT_ACCEPT_TRUE_FOR_1", help="[doctest] ignored.")
             parser.add_flag("IGNORE_EXCEPTION_DETAIL", help="[doctest] ignore the exception details.")
@@ -176,13 +181,47 @@ class PythonParser(ExampleParser):
 
         return parser
 
+    def get_extended_option_parser(self, parent_parser, **kw):
+        original_compatibility_mode = getattr(self, 'compatibility_mode', None)
+
+        # compatibility mode: True if it wasn't explicitly set
+        compatibility_mode = True if original_compatibility_mode == None \
+                                  else original_compatibility_mode
+
+        # easy cake, the parsers are already cached
+        if self._optparser_extended_by_comp_mode_cache:
+            return self._optparser_extended_by_comp_mode_cache[compatibility_mode]
+
+        cached = {}
+
+        # fake the two compatibility mode (True and False)
+        # and build an extended parser for each mode
+        self.compatibility_mode = True
+        cached[self.compatibility_mode] = ExtendOptionParserMixin.get_extended_option_parser(
+                                                self, parent_parser, **kw)
+
+        self.compatibility_mode = False
+        cached[self.compatibility_mode] = ExtendOptionParserMixin.get_extended_option_parser(
+                                                self, parent_parser, **kw)
+
+        # save the extended parsers in the cache
+        self._optparser_extended_by_comp_mode_cache = cached
+
+        # restore the compatibility mode (even if it was unset)
+        if original_compatibility_mode == None:
+            del self.compatibility_mode
+        else:
+            self.compatibility_mode = original_compatibility_mode
+
+        return cached[compatibility_mode]
+
     def _map_doctest_opts_to_byexample_opts(self):
         '''
         In compatibility mode, take all the Python doctest's options and flags
         and map them to a byexample option if possible.
         Otherwise log a message.
 
-        Also, in compatibility mode, disable any "capture" unless the ELLIPSIS
+        Also, in compatibility mode, disable any "tags" unless the ELLIPSIS
         flag is present.
 
         Return a dictionary with the mapped flags; self.options is unchanged.
@@ -199,9 +238,9 @@ class PythonParser(ExampleParser):
                 mapped['skip'] = True
 
             if options['ELLIPSIS']:
-                # enable the capture if ELLIPSIS but see also expected_from_match
+                # enable the 'tags' if ELLIPSIS but see also expected_from_match
                 # as this byexample's option is not equivalent to doctest's one
-                mapped['capture'] = True
+                mapped['tags'] = True
 
             if options['REPORT_UDIFF']:
                 mapped['diff'] = 'unified'
@@ -218,9 +257,9 @@ class PythonParser(ExampleParser):
                 log("[Note] DONT_ACCEPT_TRUE_FOR_1 flag is not supported.",
                         self.verbosity-2)
 
-        # in compatibility mode, do not capture by default [force this]
-        if self.options['py_doctest'] and 'capture' not in mapped:
-            mapped['capture'] = False
+        # in compatibility mode, do not interpret <...> by default [force this]
+        if self.options['py_doctest'] and 'tags' not in mapped:
+            mapped['tags'] = False
 
         options.unmask_default()
         return mapped
@@ -344,7 +383,7 @@ class PythonParser(ExampleParser):
                                     ])
 
                 # enable the capture, this should affect to this example only
-                options['capture'] = True
+                options['tags'] = True
 
         options.unmask_default()
         return expected_str
@@ -377,11 +416,10 @@ class PythonInterpreter(ExampleRunner, PexepctMixin):
         self._PS2 = r'/byexample/py/ps2> '
 
         PexepctMixin.__init__(self,
-                                cmd=None, # patchme later
                                 PS1_re = self._PS1,
                                 any_PS_re = r'/byexample/py/ps\d> ')
 
-    def _get_cmd(self, pretty_print):
+    def get_default_cmd(self, pretty_print, columns, *args, **kargs):
         # Important: do not use a single quote ' in the following python code
         # it will break it in real hard ways to debug.
         change_prompts = r'''
@@ -428,7 +466,7 @@ if %s:
                 # unpatch
                 self.builtins_module.repr = self.orig_repr
 
-    __byexample_pretty_print = __ByexamplePrettyPrint(indent=1, width=80, depth=None)
+    __byexample_pretty_print = __ByexamplePrettyPrint(indent=1, width=%i, depth=None)
     del __ByexamplePrettyPrint
 
     # change the displayhook to use pprint instead of repr
@@ -439,9 +477,15 @@ if %s:
 # remove introduced symbols
 del sys
 del _byexample_pprint
-''' % (self._PS1, self._PS2, pretty_print)
+''' % (self._PS1, self._PS2, pretty_print, columns)
 
-        return "/usr/bin/env python -i -c '%s'" % change_prompts
+        return  "%e %p %a", {
+                    'e': "/usr/bin/env",
+                    'p': "python",
+                    'a': [
+                            "-i", # mean interactive, run -c arg and continue running
+                            "-c", change_prompts,  # run this before anything else
+                     ]}
 
     def run(self, example, flags):
         return self._exec_and_wait(example.source,
@@ -450,17 +494,20 @@ del _byexample_pprint
     def interact(self, example, options):
         PexepctMixin.interact(self)
 
-    def initialize(self, examples, options):
+    def initialize(self, options):
         py_doctest = options.get('py_doctest', False)
         py_pretty_print = options.get('py_pretty_print', False)
         pretty_print = (py_doctest and py_pretty_print) \
                         or not py_doctest
 
-        # set the final command
-        self.cmd = self._get_cmd(pretty_print)
+        shebang, tokens = self.get_default_cmd(pretty_print, options['geometry'][1])
+        shebang = options['shebangs'].get(self.language, shebang)
+
+        cmd = ShebangTemplate(shebang).quote_and_substitute(tokens)
 
         # run!
-        self._spawn_interpreter(delaybeforesend=options['delaybeforesend'])
+        self._spawn_interpreter(cmd, delaybeforesend=options['delaybeforesend'],
+                                     geometry=options['geometry'])
 
     def shutdown(self):
         self._shutdown_interpreter()
