@@ -1,7 +1,76 @@
-import re, pexpect, sys, time, termios, operator
+import re, pexpect, sys, time, termios, operator, string, shlex, os
+from functools import reduce
 from .executor import TimeoutException
 from .common import tohuman
 
+try:
+    from shlex import quote as shlex_quote
+except ImportError:
+    from pipes import quote as shlex_quote
+
+class ShebangTemplate(string.Template):
+    delimiter = '%'
+
+    def quote_and_substitute(self, tokens):
+        '''
+        Quote each token to be suitable for shell expansion and then
+        perform a substitution in the template.
+
+        >>> from byexample.runner import ShebangTemplate
+        >>> tokens = {'a': ['-i', "-c", 'blue = "1"'],
+        ...           'e': '/usr/bin/env', 'p': 'python'}
+
+        The basic case is a simple template where each token
+        is quoted except the lists: each item is quoted but not the
+        whole list as a single unit.
+
+        >>> shebang = '%e %p %a'
+        >>> print(ShebangTemplate(shebang).quote_and_substitute(tokens))
+        /usr/bin/env python -i -c 'blue = "1"'
+
+        This works even if the token in the template are already quoted
+        >>> shebang = '/bin/sh -c \'%e %p %a >/dev/null\''
+        >>> print(ShebangTemplate(shebang).quote_and_substitute(tokens))
+        /bin/sh -c '/usr/bin/env python -i -c '"'"'blue = "1"'"'"' >/dev/null'
+
+        Here is another pair of examples:
+        >>> tokens = {'a': ['-i', "-c", 'blue = \'1\''],
+        ...           'e': '/usr/bin/env', 'p': 'py\'thon'}
+
+        >>> shebang = '%e %p %a'
+        >>> print(ShebangTemplate(shebang).quote_and_substitute(tokens))
+        /usr/bin/env 'py'"'"'thon' -i -c 'blue = '"'"'1'"'"''
+
+        >>> shebang = '/bin/sh -c \'%e %p %a >/dev/null\''
+        >>> print(ShebangTemplate(shebang).quote_and_substitute(tokens))
+        /bin/sh -c '/usr/bin/env '"'"'py'"'"'"'"'"'"'"'"'thon'"'"' -i -c '"'"'blue = '"'"'"'"'"'"'"'"'1'"'"'"'"'"'"'"'"''"'"' >/dev/null'
+        '''
+
+        self._tokens = {}
+        self._not_quote_them = []
+        for k, v in tokens.items():
+            if isinstance(v, (list, tuple)):
+                self._tokens[k] = ' '.join(shlex_quote(i) for i in v)
+            else:
+                self._tokens[k] = shlex_quote(v)
+
+        cmd = []
+        for x in shlex.split(self.template):
+            # *before* the expansion, will this require quote? (will yield
+            # more than a single item?)
+            should_quote = len(shlex.split(x)) > 1
+
+            # perform the expansion
+            x = ShebangTemplate(x).substitute(self._tokens)
+
+            # was needed to quote this *before* the expansion?
+            if should_quote:
+                x = shlex_quote(x)
+
+            cmd.append(x)
+
+        return ' '.join(cmd)
+
 class ExampleRunner(object):
     def __init__(self, verbosity, encoding, **unused):
         self.verbosity = verbosity
@@ -46,7 +115,7 @@ class ExampleRunner(object):
         '''
         raise NotImplementedError() # pragma: no cover
 
-    def initialize(self):
+    def initialize(self, options):
         '''
         Hook to initialize the runner. This method will be called
         before running any example.
@@ -60,20 +129,28 @@ class ExampleRunner(object):
         '''
         raise NotImplementedError() # pragma: no cover
 
+
 class PexepctMixin(object):
-    def __init__(self, cmd, PS1_re, any_PS_re):
-        self.cmd = cmd
+    def __init__(self, PS1_re, any_PS_re):
         self.PS1_re = re.compile(PS1_re)
         self.any_PS_re = re.compile(any_PS_re)
 
         self.last_output = []
 
-    def _spawn_interpreter(self, delaybeforesend=0.010, wait_first_prompt=True,
-                                        first_prompt_timeout=10):
+    def _spawn_interpreter(self, cmd, delaybeforesend=None,
+                                        wait_first_prompt=True,
+                                        first_prompt_timeout=10,
+                                        geometry=(24, 80)):
+        env = os.environ.copy()
+        env.update({'LINES': str(geometry[0]), 'COLUMNS': str(geometry[1])})
+
         self._drop_output() # there shouldn't be any output yet but...
-        self.interpreter = pexpect.spawn(self.cmd, echo=False,
-                                                encoding=self.encoding)
+        self.interpreter = pexpect.spawn(cmd, echo=False,
+                                                encoding=self.encoding,
+                                                dimensions=geometry,
+                                                env=env)
         self.interpreter.delaybeforesend = delaybeforesend
+        self.interpreter.delayafterread = None
 
         if wait_first_prompt:
             self._expect_prompt(timeout=first_prompt_timeout, prompt_re=self.PS1_re)
@@ -130,7 +207,9 @@ class PexepctMixin(object):
         '''
         _timeout_msg = "Prompt not found: the code is taking too long to finish or there is a syntax error.\nLast 1000 bytes read:\n%s"
         if timeout <= 0:
-            raise TimeoutException(_timeout_msg % ''.join(self.last_output)[-1000:])
+            out = self._get_output()
+            raise TimeoutException(_timeout_msg % ''.join(self.last_output)[-1000:],
+                                    out)
 
         if not prompt_re:
             prompt_re = self.any_PS_re
@@ -142,7 +221,9 @@ class PexepctMixin(object):
         self.last_output.append(self.interpreter.before)
 
         if what == Timeout:
-            raise TimeoutException(_timeout_msg % ''.join(self.last_output)[-1000:])
+            out = self._get_output()
+            raise TimeoutException(_timeout_msg % ''.join(self.last_output)[-1000:],
+                                    out)
 
 
     def _get_output(self):
